package cn.kdan.cloud.pdf.office.payment.service.impl;

import cn.hutool.core.bean.BeanUtil;
import cn.kdan.cloud.pdf.office.api.account.feign.UserApi;
import cn.kdan.cloud.pdf.office.api.account.feign.UserSubscriptionInfoApi;
import cn.kdan.cloud.pdf.office.api.account.utils.SubscriptionUtil;
import cn.kdan.cloud.pdf.office.api.account.vo.UserSubscriptionInfo;
import cn.kdan.cloud.pdf.office.api.email.bo.EmailSendBO;
import cn.kdan.cloud.pdf.office.api.email.feign.EmailApi;
import cn.kdan.cloud.pdf.office.api.payment.constant.OrderConstant;
import cn.kdan.cloud.pdf.office.api.payment.constant.SubscriptionConstant;
import cn.kdan.cloud.pdf.office.api.payment.vo.OrdersVO;
import cn.kdan.cloud.pdf.office.api.product.feign.ProductApi;
import cn.kdan.cloud.pdf.office.api.product.vo.ProductVO;
import cn.kdan.cloud.pdf.office.common.constant.CommonConstant;
import cn.kdan.cloud.pdf.office.common.dto.LicenseCodeResult;
import cn.kdan.cloud.pdf.office.common.enums.*;
import cn.kdan.cloud.pdf.office.common.enums.account.AccountTypeEnum;
import cn.kdan.cloud.pdf.office.common.enums.account.PDFOfficeUserSubscriptionPayTypeEnum;
import cn.kdan.cloud.pdf.office.common.enums.account.PDFOfficeUserSubscriptionStatusEnum;
import cn.kdan.cloud.pdf.office.common.enums.account.PayTypeEnum;
import cn.kdan.cloud.pdf.office.common.utils.CommonUtils;
import cn.kdan.cloud.pdf.office.common.utils.MyDateUtils;
import cn.kdan.cloud.pdf.office.common.utils.PDFReaderUtils;
import cn.kdan.cloud.pdf.office.common.vo.UserInfoVO;
import cn.kdan.cloud.pdf.office.common.vo.UserSubscriptionInfoVO;
import cn.kdan.cloud.pdf.office.common.vo.UserVO;
import cn.kdan.cloud.pdf.office.payment.entity.Order;
import cn.kdan.cloud.pdf.office.payment.entity.OrderGift;
import cn.kdan.cloud.pdf.office.payment.entity.RightBindLog;
import cn.kdan.cloud.pdf.office.payment.entity.Subscription;
import cn.kdan.cloud.pdf.office.payment.properties.PayCenterConfigProperties;
import cn.kdan.cloud.pdf.office.payment.service.*;
import cn.kdan.cloud.pdf.office.payment.utils.RSAUtils;
import cn.kdan.cloud.pdf.office.payment.utils.TemplatesUtil;
import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.toolkit.CollectionUtils;
import com.baomidou.mybatisplus.core.toolkit.ObjectUtils;
import com.baomidou.mybatisplus.core.toolkit.StringUtils;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import okhttp3.OkHttpClient;
import okhttp3.Request;
import okhttp3.RequestBody;
import okhttp3.Response;
import org.jetbrains.annotations.NotNull;
import org.redisson.api.RLock;
import org.redisson.api.RedissonClient;
import org.springframework.amqp.rabbit.core.RabbitTemplate;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.http.HttpEntity;
import org.springframework.http.HttpHeaders;
import org.springframework.http.MediaType;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.LinkedMultiValueMap;
import org.springframework.util.MultiValueMap;
import org.springframework.util.StreamUtils;
import org.springframework.web.client.RestTemplate;

import java.io.IOException;
import java.math.BigDecimal;
import java.nio.charset.StandardCharsets;
import java.text.SimpleDateFormat;
import java.time.Instant;
import java.time.ZoneId;
import java.util.*;
import java.util.concurrent.TimeUnit;
import java.util.stream.Collectors;

/**
 * @author ComPDFKit-WPH 2023/2/9
 */
@Service
@Slf4j
@RequiredArgsConstructor
@Transactional
public class PayCenterWebhookServiceImpl implements PayCenterWebhookService {

    private final UserApi userApi;

    private final UserSubscriptionInfoApi userSubscriptionInfoApi;

    private final ProductApi productApi;

    private final OrderService ordersService;

    private final SubscriptionsService subscriptionsService;

    private final RedissonClient redissonClient;

    private final RabbitTemplate rabbitTemplate;

    private final OrderGiftService orderGiftService;

    private final PayCenterConfigProperties payCenterConfigProperties;

    private final EmailApi emailApi;
    private final RightBindLogService rightBindLogService;
    @Value("${pdfReaderUrl:123/}")
    private String pdfReaderUrl;

    private static final RestTemplate restTemplate = new RestTemplate();

    @Value("${htmlToPdfUrl:http://139.196.160.101:3060/api/get-invoice}")
    private String htmlToPdfUrl;

    /**
     * 支付中台统一回调
     * @param  body  请求参数
     *                {
     *                 "event_type": "事件类型：1 支付成功(订阅激活成功)回调，2 支付失败回调，3 订阅扣款成功回调，4 订阅扣款失败回调，5 取消订阅回调",
     *                 "third_trade_id": "第三方订单号",
     *                 "pay_id": "第三方交易id",
     *                 "nextBillingTime": "订阅-下次扣款时间",
     *                 "order_num": "我们这边创建订单时传过去的自定义订单号",
     *                }
     * @return String 支付链接
     */
    @Override
    @Transactional
    public Boolean webhookMonitor(Map<String, String> body) {
        if (body == null || body.isEmpty()) {
            log.error("支付中台统一回调，请求参数为空");
            return false;
        }
        log.info("支付中台统一回调，请求参数：{}", body);
        String eventType = body.get("event_type");
        String thirdTradeId = body.get("third_trade_id");
        String payId = body.get("pay_id");
        String nextBillingTime = body.get("next_billing_time");
        String orderNum = body.get("order_num");
        // 获取 RLock 对象
        String lockKey = eventType.concat(thirdTradeId);
        RLock rLock = redissonClient.getLock(lockKey);
        try {
            boolean isLock = rLock.tryLock(30, 10, TimeUnit.SECONDS);
            if (!isLock) {
                log.info("获取分布式锁失败，请重试");
                return false;
            }
            // 处理回调
            // 根据第三方id获取订单
            List<OrdersVO> ordersVOList = ordersService.getOrderListByThirdTradeNo(thirdTradeId);
            if (CollectionUtils.isEmpty(ordersVOList)) {
                log.error("根据第三方订单id查询不到订单信息，thirdTradeId：{}", thirdTradeId);
                // 根据自定义订单号查询
                ordersVOList = ordersService.getOrderByOrderNum(orderNum);
                if (CollectionUtils.isEmpty(ordersVOList)) {
                    log.error("根据自定义订单号查询不到订单信息，orderNum：{}", orderNum);
                    return false;
                }
            }
            // 获取用户订阅信息
            UserInfoVO userInfoVO = userApi.getMemberInfoById(ordersVOList.get(0).getUserId()).getResult();
            List<UserSubscriptionInfoVO> subscriptionInfoList = userInfoVO.getSubscriptionInfoList();
            // 获取产品信息
            ProductVO productVO = productApi.getProduct(ordersVOList.get(0).getProductId()).getResult();
            log.info("订单回调，产品信息，{}", productVO.toString());
            // 根据回调事件
            switch (eventType) {
                case "1":
                    // 买断支付成功，订阅首期支付成功
                    // 判断order，买断和订阅首期支付不会有多条order记录
                    if (ordersVOList.size() != 1) {
                        log.error("支付回调错误！查询到多条订单信息，thirdTradeId：{}，orderNum", thirdTradeId, orderNum);
                        return false;
                    }
                    OrdersVO ordersVO = ordersVOList.get(0);
                    log.info("支付成功回调，订单：{}", ordersVO);
                    if (ordersVO.getStatus().equals(OrderConstant.COMPLETED)) {
                        log.error("支付的订单已完成，id:{}", ordersVO.getId());
                        return false;
                    }
                    // 判断关闭的订单，买断的正常走，订阅的判断用户订阅权益，不符合条件的退单和取消
                    //1、如果是永久商品，正常收款，把本地这笔订单状态由“已关闭”改成“已支付”；
                    //2、如果是订阅商品，判断用户有没有订阅权益：如果没有就正常收款，把本地这笔订单状态由“已关闭”改成“已支付”；如果有订阅权益，将这笔关闭的订单在第三方退单并取消自动续订，订单状态也由“已关闭”改成“已退款”；
                    //3、如果是0元领取AI商品，判断用户有没有订阅权益：如果有高级版年订阅权益，正常收款，把本地这笔订单状态由“已关闭”改成“已支付”；如果没有高级版年订阅权益，将这笔关闭的订单在第三方取消自动续订，订单状态也由“已关闭”改成“已退款”；
                    if (ordersVO.getStatus().equals(OrderConstant.CLOSED)) {
                        log.info("支付的订单已关闭，id:{}", ordersVO.getId());
                        if (ordersVO.getPaymentModel() == 1) {
                            ordersVO.setThirdTradeNo(thirdTradeId);
                            if ("ai-subscription-year-trail".equals(productVO.getCode())) {
                                // 0元领取AI商品，判断是否高级版年订阅权益是否正常
                                if (!subscriptionsService.judgeAiDiscount(ordersVO.getUserId())) {
                                    // 没有权益，取消用户订阅
                                    log.info("关闭的0元AI订单,订阅权益失效，取消用户订阅");
                                    boolean cancelSubscriptionResult = cancelSubscription(ordersVO);
                                    if (cancelSubscriptionResult) {
                                        Order order = new Order();
                                        order.setId(ordersVO.getId());
                                        order.setRefundAmount(BigDecimal.ZERO);
                                        order.setRefundNumber(1);
                                        order.setStatus(OrderConstant.REFUND);
                                        order.setRefundDate(new Date());
                                        ordersService.updateById(order);
                                    } else {
                                        log.error("关闭的0元AI订单，取消用户订阅失败，订单id：{}", ordersVO.getId());
                                    }
                                    break;
                                }
                            } else {
                                // pdf订阅，判断用户有没有订阅权益
                                long subscriptionInfo = subscriptionInfoList.stream().filter(s -> Objects.equals(s.getPlatform(), productVO.getPlatform()) && (s.getStatus() == 1 || s.getStatus() == 3)).count();
                                if (subscriptionInfo > 0) {
                                    // 有订阅权益，退款和取消订阅
                                    log.info("关闭的订阅订单，已有订阅权益，退款并取消订阅");
                                    boolean refundOrderResult = refundOrder(ordersVO);
                                    boolean cancelSubscriptionResult = cancelSubscription(ordersVO);
                                    if (refundOrderResult && cancelSubscriptionResult) {
                                        BigDecimal refundAmount = ObjectUtils.isNotEmpty(ordersVO.getReducedPrice()) ? ordersVO.getReducedPrice() : ordersVO.getPrice();
                                        Order order = new Order();
                                        order.setId(ordersVO.getId());
                                        order.setRefundAmount(refundAmount);
                                        order.setRefundNumber(1);
                                        order.setStatus(OrderConstant.REFUND);
                                        order.setRefundDate(new Date());
                                        ordersService.updateById(order);
                                    } else {
                                        log.error("关闭的订阅订单退款或取消失败，订单：{}", ordersVO);
                                    }
                                    break;
                                }
                            }
                        }
                    }
                    // 插入subscription
                    Subscription subscription = new Subscription();
                    subscription.setId(CommonUtils.generateId());
                    subscription.setProductId(ordersVO.getProductId());
                    subscription.setStatus(SubscriptionConstant.actived);
                    if(ObjectUtils.isNotEmpty(ordersVO.getReducedPrice())){
                        subscription.setPrice(ordersVO.getReducedPrice());
                    }else{
                        subscription.setPrice(ordersVO.getPrice());
                    }
                    subscription.setUserId(ordersVO.getUserId());
                    subscription.setEmail(ordersVO.getEmail());
                    subscription.setSubscriptionType(ordersVO.getSubscriptionType());
                    subscription.setAppId(ordersVO.getAppId());
                    subscription.setPaymentModel(ordersVO.getPaymentModel());
                    subscription.setThirdSubscriptionId(thirdTradeId);
                    subscription.setOrderId(ordersVO.getId());
                    subscription.setPayment(ordersVO.getPayment());
                    subscription.setBuyerType(2);
                    log.info("用户{}，判断升级前的产品code：{},是否满足判断条件{}",userInfoVO.getEmail(),productVO.getCode(),((productVO.getCode().contains("advanced-permanent")&&productVO.getCode().contains("upgrade"))||
                            (productVO.getCode().contains("advanced-annual-subscription")&&productVO.getCode().contains("upgrade"))));
                    //如果是升级到全平台年订阅，则绑定用户账号的序列码
                    if((productVO.getCode().contains("advanced-permanent")&&productVO.getCode().contains("upgrade"))||
                            (productVO.getCode().contains("advanced-annual-subscription")&&productVO.getCode().contains("upgrade"))){
                        //默认双平台
                        int platformCount = 2;
                        String levels = ProductLevelEnum.PREMIUM.getCode();
                        if(productVO.getCode().contains("mac")||productVO.getCode().contains("windows")||productVO.getCode().contains("single")||productVO.getCode().contains("advanced-annual-subscription-standard-year-upgrade")){
                            //说明是单平台升级
                            log.info("");
                            platformCount = 1;
                        }
                        if(productVO.getCode().contains("advanced-annual-subscription-standard-year-upgrade")){
                            //说明是单平台升级
                            platformCount = 4;
                        }
                        if(productVO.getCode().contains("advanced-annual-subscription-standard-upgrade")){
                            //说明是单平台升级
                            platformCount = 1;
                        }
                        //说明原来的权益是标准
                        if(productVO.getCode().contains("standard")){
                            levels = ProductLevelEnum.STANDARD.getCode();
                        }
                        UserSubscriptionInfoVO active = CommonUtils.findActiveSubscription(subscriptionInfoList,null);
                        if(!(active.getLevels().equals(levels)&&CommonUtils.getPlatforms(active)==platformCount)){
                            transfer(ordersVO, productVO, userInfoVO);
                        }
                    }
                    // 标准订阅升级，去除原有永久版权益
                    if(productVO.getCode().contains("advanced-annual-subscription-standard-year-upgrade")) {
                        // 标准订阅升级，去除原有标准版权益
                        log.info("升级code：{}，去除原有标准版权益", productVO.getCode());
                        List<UserSubscriptionInfoVO> collect = subscriptionInfoList.stream().filter(
                                userSubscriptionInfoVO -> userSubscriptionInfoVO.getPlatform().equals(productVO.getPlatform())
                        ).collect(Collectors.toList());
                        UserSubscriptionInfoVO userSubscriptionInfoVO = collect.get(0);
                        if (userSubscriptionInfoVO.getPlatform() == 2 && userSubscriptionInfoVO.getPayType() == 1) {
                            // 标准订阅升级，取消订阅，获取用户最新的标准版年订阅订单
                            OrdersVO order = ordersService.getOrderByUserIdAndPlatform(ordersVO.getUserId(), 2);
                            log.info("标准订阅升级，取消订阅，获取用户最新的标准版年订阅订单，{}", order);
                            cancelSubscription(order);
                        }
                        log.info("去除原有标准版权益，{}", userSubscriptionInfoVO);
                        userSubscriptionInfoApi.delete(userSubscriptionInfoVO.getId());
                        subscriptionInfoList = userApi.getMemberInfoById(ordersVO.getUserId()).getResult().getSubscriptionInfoList();
                    }
                    // 订阅订单，设置订阅开始时间和结束时间
                    if (ordersVO.getPaymentModel() == 1) {
                        // 获取当前时间
                        Date currentDate = new Date();
                        subscription.setStartDate(currentDate);
                        Date endDate ;
                        if(productVO.getCode().contains("annual-subscription-trail")){
                            Calendar calendar = Calendar.getInstance();
                            calendar.setTime(currentDate);
                            calendar.add(Calendar.DAY_OF_MONTH, 7);
                            endDate = calendar.getTime();
                        }else{
                            endDate = getDateByCycle(currentDate, productVO.getCycle());
                        }
                        subscription.setEndDate(endDate);
                        subscriptionsService.save(subscription);
                        // 发送订阅过期消息
                        SubscriptionUtil.subscriptionExpiredSendNews(rabbitTemplate, subscription.getId(), endDate.toInstant().atZone(ZoneId.systemDefault()).toLocalDateTime());
                    }
                    // 更新订单状态
                    Order order = new Order();
                    order.setId(ordersVO.getId());
                    order.setSubscriptionId(subscription.getId());
                    order.setThirdOrderNo(payId);
                    order.setThirdTradeNo(thirdTradeId);
                    order.setStatus(OrderConstant.COMPLETED);
                    order.setPayDate(new Date());
                    order.setInvoiceNo(MyDateUtils.getTimeStamp() + (int) ((Math.random() * 9 + 1) * 1000));
                    ordersService.updateById(order);
                    ordersVO.setInvoiceNo(order.getInvoiceNo());
                    // 更新user_subscription_info
                    UserSubscriptionInfo userSubscriptionInfo = new UserSubscriptionInfo();

                    boolean emailFlag = false;
                    // 如果是批量的购买，不需要判断用户有没有购买，直接存到订单赠送
                    if (productVO.getPlatform() == 1 && ordersVO.getPayNumber() > 1) {
                        log.info("批量购买{}个，新增订单赠送", ordersVO.getPayNumber());
                        List<OrderGift> orderGiftList = new ArrayList<>();
                        for (Integer i = 0; i < ordersVO.getPayNumber(); i++) {
                            OrderGift orderGift = new OrderGift();
                            orderGift.setId(CommonUtils.generateId());
                            orderGift.setOrderId(ordersVO.getId());
                            orderGift.setProductId(ordersVO.getProductId());
                            orderGift.setUserId(ordersVO.getUserId());
                            orderGift.setValidFlag("1");
                            orderGiftList.add(orderGift);
                        }
                        orderGiftService.saveBatch(orderGiftList);
                    } else {
                        // 查询用户产品类型的订阅信息
                        List<UserSubscriptionInfoVO> collect = subscriptionInfoList.stream().filter(
                                userSubscriptionInfoVO -> userSubscriptionInfoVO.getPlatform().equals(productVO.getPlatform())
                        ).collect(Collectors.toList());
                        if (CollectionUtils.isNotEmpty(collect)) {
                            log.info("用户有订阅信息");
                            if (productVO.getPlatform() == 1) {
                                // win、mac单平台标准永久升级高级永久，新增订阅记录
                                if (productVO.getCode().contains("advanced-permanent-windows-standard-upgrade") || productVO.getCode().contains("advanced-permanent-mac-upgrade")) {
                                    log.info("win、mac单平台标准永久升级高级永久，新增订阅记录");
                                    userSubscriptionInfo.setId(CommonUtils.generateId());
                                    userSubscriptionInfo.setUserId(ordersVO.getUserId());
                                    userSubscriptionInfo.setStatus(PDFOfficeUserSubscriptionStatusEnum.SUBSCRIPTION_IN_PROGRESS.value());
                                    userSubscriptionInfo.setAppId(ordersVO.getAppId());
                                    userSubscriptionInfo.setProductId(ordersVO.getProductId());
                                    userSubscriptionInfo.setPlatform(productVO.getPlatform());
                                    // 设置过期时间一百年
                                    // 获取当前时间
                                    Calendar calendar = Calendar.getInstance();
                                    // 向当前时间增加100年
                                    calendar.add(Calendar.YEAR, 100);
                                    userSubscriptionInfo.setEndDate(calendar.getTime());
                                    userSubscriptionInfo.setPayType(0);
                                    subscription.setStartDate(new Date());
                                    subscription.setEndDate(calendar.getTime());
                                    subscriptionsService.save(subscription);
                                    userSubscriptionInfoApi.insert(userSubscriptionInfo);
                                    emailFlag = true;
                                } else {
                                    log.info("永久版产品:{}，新增订单赠送", ordersVO.getId());
                                    OrderGift orderGift = new OrderGift();
                                    orderGift.setId(CommonUtils.generateId());
                                    orderGift.setOrderId(ordersVO.getId());
                                    orderGift.setProductId(ordersVO.getProductId());
                                    orderGift.setUserId(ordersVO.getUserId());
                                    orderGift.setValidFlag("1");
                                    orderGiftService.save(orderGift);
                                }
                            } else {
                                log.info("订阅版产品，更新用户订阅信息");
                                UserSubscriptionInfoVO userSubscriptionInfoVO = collect.get(0);
                                BeanUtil.copyProperties(userSubscriptionInfoVO, userSubscriptionInfo);
                                userSubscriptionInfo.setStatus(PDFOfficeUserSubscriptionStatusEnum.SUBSCRIPTION_IN_PROGRESS.value());
                                // 获取当前时间
                                Date currentDate = new Date();
                                // 根据产品周期添加订阅信息记录的结束时间
                                if (ObjectUtils.isNotEmpty(userSubscriptionInfoVO.getEndDate())&&
                                        currentDate.compareTo(userSubscriptionInfoVO.getEndDate()) < 0) {
                                    // 订阅未过期，设置结束时间为当前时间加产品周期
                                    currentDate = userSubscriptionInfoVO.getEndDate();
                                }
                                Date endDate = getDateByCycle(currentDate, productVO.getCycle());
                                userSubscriptionInfo.setEndDate(endDate);
                                userSubscriptionInfo.setPayType(1);
                                userSubscriptionInfo.setPlatform(productVO.getPlatform());
                                userSubscriptionInfo.setAppId(ordersVO.getAppId());
                                userSubscriptionInfo.setProductId(ordersVO.getProductId());
                                UserVO userVO = new UserVO();
                                userVO.setId(ordersVO.getUserId());
                                if(productVO.getPlatform()==3){
                                    //如果是ai还要加ai点数
                                    userSubscriptionInfo.setPoint(userSubscriptionInfo.getPoint()+Integer.valueOf(productVO.getPoints()));
                                    if(productVO.getCode().contains("trail")){
                                        userVO.setCanAccessAi("0");
                                        userApi.updateUser(userVO);
                                    }
                                } else {
                                    userVO.setAccountType(AccountTypeEnum.PAID_ACCOUNT.value());
                                    userApi.updateUser(userVO);
                                }
                                userSubscriptionInfoApi.update(userSubscriptionInfo);
                                emailFlag = true;
                            }
                        } else {
                            log.info("用户没有订阅信息，新增记录");
                            userSubscriptionInfo.setId(CommonUtils.generateId());
                            userSubscriptionInfo.setUserId(ordersVO.getUserId());
                            userSubscriptionInfo.setStatus(PDFOfficeUserSubscriptionStatusEnum.SUBSCRIPTION_IN_PROGRESS.value());
                            userSubscriptionInfo.setAppId(ordersVO.getAppId());
                            userSubscriptionInfo.setProductId(ordersVO.getProductId());
                            //0免费，1永久版，2订阅，3ai
                            userSubscriptionInfo.setPlatform(productVO.getPlatform());
                            if (productVO.getPlatform() == 1) {
                                // 设置过期时间一百年
                                // 获取当前时间
                                Calendar calendar = Calendar.getInstance();
                                // 向当前时间增加100年
                                calendar.add(Calendar.YEAR, 100);
                                userSubscriptionInfo.setEndDate(calendar.getTime());
                                userSubscriptionInfo.setPayType(0);

                                subscription.setStartDate(new Date());
                                subscription.setEndDate(calendar.getTime());
                                subscriptionsService.save(subscription);
                            } else {
                                Date endDate;
                                if(productVO.getCode().contains("advanced-annual-subscription-standard-year-upgrade")) {
                                    endDate = Date.from(Instant.parse(nextBillingTime));
                                } else {
                                    // 获取当前时间
                                    Date currentDate = new Date();
                                    if(productVO.getCode().contains("annual-subscription-trail")){
                                        Calendar calendar = Calendar.getInstance();
                                        calendar.setTime(currentDate);
                                        calendar.add(Calendar.DAY_OF_MONTH, 7);
                                        endDate = calendar.getTime();
                                    }else{
                                        endDate = getDateByCycle(currentDate, productVO.getCycle());
                                    }
                                }
                                userSubscriptionInfo.setEndDate(endDate);
                                userSubscriptionInfo.setPayType(1);
                            }
                            UserVO userVO = new UserVO();
                            userVO.setId(ordersVO.getUserId());
                            if(productVO.getPlatform()==3){
                                //如果是ai还要加ai点数
                                userSubscriptionInfo.setPoint(Integer.valueOf(productVO.getPoints()));
                                if(productVO.getCode().contains("trail")){
                                    userVO.setCanAccessAi("0");
                                    userApi.updateUser(userVO);
                                }
                            } else {
                                userVO.setAccountType(AccountTypeEnum.PAID_ACCOUNT.value());
                                userApi.updateUser(userVO);
                            }
                            userSubscriptionInfoApi.insert(userSubscriptionInfo);
                            emailFlag = true;
                        }
                    }
                    // 活动购买的高级版年订阅,双平台永久,赠送50点AI
                    if (productVO.getCode().contains(ProductCodeEnum.ADVANCED_PERMANENT_BLACKFIVE.getCode()) || productVO.getCode().contains(ProductCodeEnum.ADVANCED_ANNUAL_SUBSCRIPTION_BLACKFIVE.getCode())) {
                        log.info("购买商品code：{},赠送50点AI", productVO.getCode());
                        // 获取50点ai赠送商品
                        ProductVO productGift = productApi.getProductByCode(ProductCodeEnum.AI_GIVE_50.getCode()).getResult();
                        // 查询用户产品类型的订阅信息
                        List<UserSubscriptionInfoVO> aiUserSubscriptionInfoList = subscriptionInfoList.stream().filter(s -> s.getPlatform() == 3).collect(Collectors.toList());
                        UserSubscriptionInfo aiUserSubscriptionInfo = new UserSubscriptionInfo();
                        aiUserSubscriptionInfo.setStatus(PDFOfficeUserSubscriptionStatusEnum.SUBSCRIPTION_IN_PROGRESS.value());
                        aiUserSubscriptionInfo.setAppId(ordersVO.getAppId());
                        aiUserSubscriptionInfo.setProductId(productGift.getId());
                        if (CollectionUtils.isNotEmpty(aiUserSubscriptionInfoList)) {
                            UserSubscriptionInfoVO userSubscriptionInfoVO = aiUserSubscriptionInfoList.get(0);
                            BeanUtil.copyProperties(userSubscriptionInfoVO, aiUserSubscriptionInfo);
                            if (ObjectUtils.isNotEmpty(aiUserSubscriptionInfo.getPoint()) && aiUserSubscriptionInfo.getPoint() >= 0) {
                                aiUserSubscriptionInfo.setPoint(aiUserSubscriptionInfo.getPoint() + Integer.parseInt(productGift.getPoints()));
                            }
                            userSubscriptionInfoApi.update(aiUserSubscriptionInfo);
                        } else {
                            aiUserSubscriptionInfo.setId(CommonUtils.generateId());
                            aiUserSubscriptionInfo.setUserId(ordersVO.getUserId());
                            aiUserSubscriptionInfo.setPlatform(3);
                            aiUserSubscriptionInfo.setEndDate(getDateByCycle(new Date(), 4));
                            aiUserSubscriptionInfo.setPayType(0);
                            aiUserSubscriptionInfo.setPoint(Integer.parseInt(productGift.getPoints()));
                            userSubscriptionInfoApi.insert(aiUserSubscriptionInfo);
                        }
                    }
                    //删除缓存中的订单
                    ordersService.deleteTheCacheInTheCacheManager(ordersVO.getUserId() + "-" + ordersVO.getProductId() + "-" + ordersVO.getPayment() + "-" + ordersVO.getPayNumber() + "-" + ordersVO.getDiscountType());
                    // 关闭用户同类型订单
                    ordersService.closeOrderByUser(ordersVO.getUserId(), productVO.getPlatform());
                    if (emailFlag) {
                        log.info("用户订阅或支付成功，发送订阅邮件");
                        EmailSendBO bo = new EmailSendBO();
                        bo.setToEmail(userInfoVO.getEmail());
                        //设置事件
                        if (ordersVO.getDiscountType() == 2) {
                            bo.setUseEvent(EmailCodeTypeEnum.MEMBER_UPGRADE_SUCCESS.value());
                        } else {
                            bo.setUseEvent(EmailCodeTypeEnum.MEMBER_BUY_SUCCESS.value());
                        }
                        //设置title
                        Map<String,String> titleMap = new HashMap<>();
                        bo.setSendTitleContent(titleMap);
                        //设置内容
                        Map<String,String> contentMap = new HashMap<>();
                        contentMap.put("@plan@", productVO.getProductName());
                        Date date = new Date();
                        // 创建 SimpleDateFormat 对象，指定日期格式
                        SimpleDateFormat sdf = new SimpleDateFormat("dd/MM/yyyy");
                        // 格式化当前日期
                        String formattedDate = sdf.format(date);
                        contentMap.put("@date@",formattedDate);
                        String price = ObjectUtils.isNotEmpty(ordersVO.getReducedPrice())? ordersVO.getReducedPrice().toEngineeringString() : ordersVO.getPrice().toEngineeringString();
                        String currencySign;
                        if (ordersVO.getPayment() == 1 || ordersVO.getPayment() == 2) {
                            currencySign = "CNY";
                        } else {
                            currencySign = "USD";
                        }
                        contentMap.put("@payPrice@", currencySign + price);
                        contentMap.put("@renewPrice@", currencySign + price);
                        bo.setSendContent(contentMap);
                        log.info("获取发票");
                        String invoiceHtml = null;
                        try {
                            invoiceHtml = new String(StreamUtils.copyToByteArray(this.getClass().getClassLoader().getResourceAsStream("templates/invoice_member.html")), StandardCharsets.UTF_8);
                        } catch (IOException e) {
                            log.error(e.getMessage());
                        }
                        Map<String, Object> map = new HashMap<>();
                        map.put("Name", userInfoVO.getEmail());
                        map.put("email", userInfoVO.getEmail());
                        map.put("purchasedDate", formattedDate);
                        map.put("orderNumber", ordersVO.getTradeNo());
                        map.put("invoiceDate", formattedDate);
                        map.put("invoiceNumber", ordersVO.getInvoiceNo());
                        map.put("quantity", ordersVO.getPayNumber());
                        BigDecimal unitPrice;
                        if (ordersVO.getPayment() == 1 || ordersVO.getPayment() == 2) {
                            unitPrice = productVO.getCnyPrice();
                        } else {
                            unitPrice = productVO.getPrice();
                        }
                        map.put("unitPrice", unitPrice.toEngineeringString());
                        String discount = "0";
                        String amount = ordersVO.getPrice().toEngineeringString();
                        if (ordersVO.getReducedPrice() != null) {
                            /*if (ordersVO.getPayNumber() > 0) {
                                BigDecimal totalPrice = productVO.getPrice().multiply(BigDecimal.valueOf(ordersVO.getPayNumber()));
                                discount = totalPrice.subtract(ordersVO.getReducedPrice()).toEngineeringString();
                            } else {
                                discount = productVO.getPrice().subtract(ordersVO.getReducedPrice()).toEngineeringString();
                            }*/
                            discount = unitPrice.subtract(ordersVO.getReducedPrice()).toEngineeringString();
                            amount = ordersVO.getReducedPrice().toEngineeringString();
                        }
                        map.put("discount", discount);
                        map.put("amount", amount);
                        map.put("total", amount);
                        log.info("发票html内容填充map:{}", map);
                        // 生成替换内容后的发票
                        String htmlStr = TemplatesUtil.replaceStringUsingFreeMarker(invoiceHtml, map);
                        // 微信支付宝币种改为 CNY
                        if (ordersVO.getPayment() == 1 || ordersVO.getPayment() == 2) {
                            htmlStr = htmlStr.replace("USD", "CNY");
                        }
                        HttpHeaders headers = new HttpHeaders();
                        headers.setContentType(MediaType.APPLICATION_FORM_URLENCODED);
                        // 封装表单数据
                        MultiValueMap<String, String> paramMap = new LinkedMultiValueMap<>();
                        paramMap.add("html", htmlStr);
                        HttpEntity<MultiValueMap<String, String>> request = new HttpEntity<>(paramMap, headers);
                        // 发送POST请求,html转pdf
                        Map postForObjectMap = restTemplate.postForObject(htmlToPdfUrl, request, Map.class);
                        String invoiceFileUrl = postForObjectMap.get("url").toString();
                        log.info("发票文件地址：{}", invoiceFileUrl);
                        bo.setFileUrl(invoiceFileUrl);
                        bo.setFileName("invoice.pdf");
                        log.info("发送订阅邮件");
                        emailApi.sendEmail(bo);
                    }
                    break;
                case "2":
                    // 支付失败，有延时队列暂不处理
                    log.info("支付失败，有延时队列暂不处理，thirdTradeId：{}", thirdTradeId);
                    break;
                case "3":
                    // 订阅周期扣款成功
                    log.info("订阅周期扣款，订单：{}", ordersVOList.get(0));
                    // 新增order
                    Order cycleOrder = new Order();
                    BeanUtil.copyProperties(ordersVOList.get(0), cycleOrder);
                    cycleOrder.setId(CommonUtils.generateId());
                    cycleOrder.setTradeNo(cn.kdan.cloud.pdf.office.payment.utils.CommonUtils.generateRightsId((long) ((Math.random() * 9999 + 1) * 10000)));
                    cycleOrder.setThirdOrderNo(payId);
                    cycleOrder.setReducedPrice(null);
                    cycleOrder.setPayDate(new Date());
                    ordersService.save(cycleOrder);
                    // 通过thirdTradeId更新subscription
                    // 新增subscription
                    Subscription subscriptionVO = subscriptionsService.getOne(new LambdaQueryWrapper<Subscription>().eq(Subscription::getOrderId, ordersVOList.get(0).getId()));
                    Subscription serviceOne = new Subscription();
                    BeanUtil.copyProperties(subscriptionVO, serviceOne);
                    serviceOne.setId(CommonUtils.generateId());
                    serviceOne.setOrderId(cycleOrder.getId());
                    serviceOne.setPrice(cycleOrder.getPrice());
                    serviceOne.setStatus(SubscriptionConstant.actived);
                    // 更新user_subscription_info
                    UserSubscriptionInfo userSubscriptionInfo1 = new UserSubscriptionInfo();
                    // 查询用户订阅的信息
                    List<UserSubscriptionInfoVO> collect = subscriptionInfoList.stream().filter(userSubscriptionInfoVO -> userSubscriptionInfoVO.getPlatform().equals(productVO.getPlatform())).collect(Collectors.toList());
                    if (CollectionUtils.isEmpty(collect)) {
                        log.error("订阅周期付款，用户订阅信息异常，thirdTradeId：{}", thirdTradeId);
                        break;
                    }
                    BeanUtil.copyProperties(collect.get(0), userSubscriptionInfo1);
                    // 获取当前时间
                    Date currentDate = new Date();
                    // 根据产品周期添加订阅信息记录的结束时间
                    if (currentDate.compareTo(userSubscriptionInfo1.getEndDate()) < 0) {
                        // 订阅未过期，设置结束时间为当前时间加产品周期
                        currentDate = userSubscriptionInfo1.getEndDate();
                    }
                    serviceOne.setStartDate(currentDate);
                    Date endDate = getDateByCycle(currentDate, productVO.getCycle());
                    serviceOne.setEndDate(endDate);
                    userSubscriptionInfo1.setStatus(PDFOfficeUserSubscriptionStatusEnum.SUBSCRIPTION_IN_PROGRESS.value());
                    userSubscriptionInfo1.setPayType(PDFOfficeUserSubscriptionPayTypeEnum.AUTOMATIC_SUBSCRIPTION.value());
                    userSubscriptionInfo1.setEndDate(endDate);
                    if(productVO.getPlatform()==3){
                        //如果是ai还要加ai点数
                        userSubscriptionInfo1.setPoint(userSubscriptionInfo1.getPoint()+Integer.parseInt(productVO.getPoints()));
                    }
                    subscriptionsService.save(serviceOne);
                    userSubscriptionInfoApi.update(userSubscriptionInfo1);
                    // 订阅过期信息
                    // 发送订阅过期消息
                    SubscriptionUtil.subscriptionExpiredSendNews(rabbitTemplate, userSubscriptionInfo1.getId(), endDate.toInstant().atZone(ZoneId.systemDefault()).toLocalDateTime());
                    break;
                case "4":
                    // 订阅扣款失败，有延时队列暂不处理
                    log.info("订阅扣款失败，有延时队列暂不处理，thirdTradeId：{}", thirdTradeId);
                    break;
                case "5":
                    // 取消订阅，更新subscription
                    log.info("用户取消订阅，更新subscription，thirdTradeId：{}", thirdTradeId);
                    // 通过thirdTradeId查询
                    Subscription serviceOne1 = subscriptionsService.getOne(new LambdaQueryWrapper<Subscription>().eq(Subscription::getOrderId, ordersVOList.get(0).getId()));
                    // 查询用户订阅的信息
                    List<UserSubscriptionInfoVO> collect1 = subscriptionInfoList.stream().filter(userSubscriptionInfoVO -> userSubscriptionInfoVO.getPlatform().equals(productVO.getPlatform())).collect(Collectors.toList());
                    if (CollectionUtils.isEmpty(collect1)) {
                        log.error("订阅取消，用户订阅信息异常，thirdTradeId：{}", thirdTradeId);
                        break;
                    }
                    // 判断取消的产品id和现有权益的产品id是不是一样
                    if (!serviceOne1.getProductId().equals(collect1.get(0).getProductId())) {
                        log.info("订阅取消，现有用户订阅产品id不一致，thirdTradeId：{}", thirdTradeId);
                        break;
                    }
                    // 更新subscription
                    Subscription subscription1 = new Subscription();
                    subscription1.setId(serviceOne1.getId());
                    subscription1.setSubscriptionType(0);
                    subscriptionsService.updateById(subscription1);
                    // 更新user_subscription_info
                    log.info("用户取消订阅信息：{}", collect1.get(0));
                    UserSubscriptionInfo userSubscriptionInfo2 = new UserSubscriptionInfo();
                    userSubscriptionInfo2.setId(collect1.get(0).getId());
                    userSubscriptionInfo2.setPayType(PDFOfficeUserSubscriptionPayTypeEnum.NON_AUTO_SUBSCRIPTION.value());
                    userSubscriptionInfoApi.update(userSubscriptionInfo2);
                    break;
                default:
                    break;
            }
        } catch (Exception e) {
            log.error("获取分布式锁异常，{}", e.getStackTrace());
            return false;
        } finally {
            if (rLock.isLocked() && rLock.isHeldByCurrentThread()) {
                rLock.unlock();
            }
        }
        return true;
    }

    private void transfer(OrdersVO ordersVO, ProductVO productVO, UserInfoVO userInfoVO) {
        List<LicenseCodeResult> licenseCodes = PDFReaderUtils.getLicenseByEmail(pdfReaderUrl, ordersVO.getEmail());
        if(licenseCodes.size()==1){
            //如果是永久版，status为1
            String productCode = CodeConvertEnum.getNewCodeByOldCode(licenseCodes.get(0).getCode());
            ProductVO product = productApi.getProductByCode(productCode).getResult();
            //插入订阅
            UserSubscriptionInfo newInfo = new UserSubscriptionInfo().withId(CommonUtils.generateId())
                    //status 已注册
                    .withUserId(ordersVO.getUserId()).withStatus(1).withAppId(CommonConstant.PDF_READER_PRO_APP_ID).withPlatform(product.getPlatform()).withPayType(PayTypeEnum.NO_AUTO.value()).withProductId(product.getId().toString());
            userSubscriptionInfoApi.insert(newInfo);
            PDFReaderUtils.invalid(pdfReaderUrl,licenseCodes.get(0).getCdkey());
            RightBindLog rightBindLogDTO = new RightBindLog();
            rightBindLogDTO.setId(CommonUtils.generateId());
            rightBindLogDTO.setCdkey(licenseCodes.get(0).getCdkey());
            rightBindLogDTO.setLicenseProductCode(licenseCodes.get(0).getCode());
            rightBindLogDTO.setProductId(product.getId());
            rightBindLogDTO.setType("1");
            rightBindLogDTO.setUserId(userInfoVO.getId());
            rightBindLogService.save(rightBindLogDTO);
            EmailSendBO bo = new EmailSendBO();
            if (!StringUtils.isEmpty(ordersVO.getModel())&& ordersVO.getModel().contains("OFFICIAL")){
                bo.setUseEvent(EmailCodeTypeEnum.TRANSFER_SUCCESS_WEB.value());
            }else {
                bo.setUseEvent(EmailCodeTypeEnum.TRANSFER_SUCCESS_APP.value());
            }
            //设置title
            Map<String,String> titleMap = new HashMap<>();
            //设置内容
            Map<String,String> contentMap = new HashMap<>();
            contentMap.put("@email@", userInfoVO.getEmail());
            bo.setSendTitleContent(titleMap);
            //设置内容
            bo.setSendContent(contentMap);
            bo.setToEmail(userInfoVO.getEmail());
            emailApi.sendEmail(bo);
        }
    }

    private boolean refundOrder(OrdersVO order) {
        String configId = null;
        switch (order.getPayment()) {
            case 0:
                // paypal
                configId = payCenterConfigProperties.getPaypal();
                break;
            case 1:
                // 支付宝
                configId = payCenterConfigProperties.getAlipay();
                break;
            case 2:
                // 微信
                configId = payCenterConfigProperties.getVx();
                break;
            case 3:
                // paddle
                configId = payCenterConfigProperties.getPaddle();
                break;
            default:
                log.error("订单退款：未知的订单支付方式，{}", order);
        }
        if (!StringUtils.isEmpty(configId)) {
            RSAUtils rsaUtils = new RSAUtils();
            String authorization = rsaUtils.encrypt(String.format("{\"configId\":\"%s\"}", configId));

            JSONObject jsonObject1 = new JSONObject();
            jsonObject1.put("orderNum", order.getTradeNo());
            // 请求支付中台
            log.info("订单退款，请求支付中台，{}", jsonObject1);
            OkHttpClient client1 = new OkHttpClient().newBuilder()
                    .connectTimeout(300, TimeUnit.SECONDS)
                    .readTimeout(300, TimeUnit.SECONDS)
                    .writeTimeout(300, TimeUnit.SECONDS)
                    .build();
            okhttp3.MediaType mediaType1 = okhttp3.MediaType.parse("application/json");
            RequestBody body1 = RequestBody.create(mediaType1, jsonObject1.toString());
            Request request1 = new Request.Builder()
                    .url(payCenterConfigProperties.getPayCenterUrl() + payCenterConfigProperties.getRefundOrderMethod())
                    .method("POST", body1)
                    .addHeader("Authorization", authorization.replace("\r", "").replace("\n", ""))
                    .addHeader("Content-Type", "application/json")
                    .addHeader("Accept", "*/*")
                    .addHeader("Connection", "keep-alive")
                    .build();
            try {
                Response response = client1.newCall(request1).execute();
                if (response.isSuccessful()) {
                    // 解析响应
                    String responseBody = response.body().string();
                    JSONObject jsonResponse = JSON.parseObject(responseBody);
                    log.info("订单退款：支付中台响应，{}", jsonResponse);
                } else {
                    log.error("订单退款：支付中台调用失败，{}", response.body().string());
                    return false;
                }
            } catch (IOException e) {
                log.error("标准订阅升级：请求支付中台取消订阅失败", e);
                return false;
            }
        }
        return true;
    }

    private boolean cancelSubscription(OrdersVO order) {
        String configId = null;
        switch (order.getPayment()) {
            case 0:
                // paypal
                configId = payCenterConfigProperties.getPaypal();
                break;
            case 3:
                // paddle
                configId = payCenterConfigProperties.getPaddle();
                break;
            default:
                log.error("订阅取消：未知的订单支付方式，{}", order);
        }
        if (!StringUtils.isEmpty(configId)) {
            RSAUtils rsaUtils = new RSAUtils();
            String authorization = rsaUtils.encrypt(String.format("{\"configId\":\"%s\"}", configId));
            JSONObject jsonObject1 = new JSONObject();
            jsonObject1.put("orderNum", order.getTradeNo());
            jsonObject1.put("tradeId", order.getThirdTradeNo());
            // 请求支付中台
            log.info("取消订阅，请求支付中台，{}", jsonObject1);
            OkHttpClient client1 = new OkHttpClient().newBuilder()
                    .connectTimeout(300, TimeUnit.SECONDS)
                    .readTimeout(300, TimeUnit.SECONDS)
                    .writeTimeout(300, TimeUnit.SECONDS)
                    .build();
            okhttp3.MediaType mediaType1 = okhttp3.MediaType.parse("application/json");
            RequestBody body1 = RequestBody.create(mediaType1, jsonObject1.toString());
            Request request1 = new Request.Builder()
                    .url(payCenterConfigProperties.getPayCenterUrl() + payCenterConfigProperties.getCancelSubscriptionMethod())
                    .method("POST", body1)
                    .addHeader("Authorization", authorization.replace("\r", "").replace("\n", ""))
                    .addHeader("Content-Type", "application/json")
                    .addHeader("Accept", "*/*")
                    .addHeader("Connection", "keep-alive")
                    .build();
            try {
                Response response = client1.newCall(request1).execute();
                if (response.isSuccessful()) {
                    // 解析响应
                    String responseBody = response.body().string();
                    JSONObject jsonResponse = JSON.parseObject(responseBody);
                    log.info("支付中台取消订阅响应，{}", jsonResponse);
                } else {
                    log.error("支付中台取消订阅失败，{}", response.body().string());
                    return false;
                }
            } catch (IOException e) {
                log.error("请求支付中台取消订阅失败", e);
                return false;
            }
        }
        return true;
    }

    @NotNull
    private static Date getDateByCycle(Date startDate, Integer cycle) {
        // 创建 Calendar 实例并设置为当前时间
        Calendar calendar = Calendar.getInstance();
        calendar.setTime(startDate);
        // 根据产品周期设置结束时间
        switch (cycle) {
            case 1:
                calendar.add(Calendar.MONTH, 1);
                break;
            case 2:
                calendar.add(Calendar.MONTH, 3);
                break;
            case 3:
                calendar.add(Calendar.MONTH, 6);
                break;
            case 4:
                calendar.add(Calendar.YEAR, 1);
                break;
        }
        return calendar.getTime();
    }

}
